@@ -1,7 +1,7 @@
 /*
  * print.c	Routines to print stuff.
  *
- * Version:	$Id: print.c,v 1.56 2008/03/28 10:10:12 aland Exp $
+ * Version:	$Id$
  *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Lesser General Public
@@ -21,7 +21,7 @@
  */
 
 #include	<freeradius-devel/ident.h>
-RCSID("$Id: print.c,v 1.56 2008/03/28 10:10:12 aland Exp $")
+RCSID("$Id$")
 
 #include	<freeradius-devel/libradius.h>
 
@@ -127,7 +127,7 @@ static int utf8_char(uint8_t *str)
  *	has to be larger than the input string by at least 5 bytes.
  *	If not, the output is silently truncated...
  */
-void librad_safeprint(char *in, size_t inlen, char *out, size_t outlen)
+void fr_print_string(char *in, size_t inlen, char *out, size_t outlen)
 {
 	uint8_t		*str = (uint8_t *)in;
 	int		sp = 0;
@@ -217,13 +217,13 @@ int vp_prints_value(char * out, size_t outlen, VALUE_PAIR *vp, int delimitst)
 			if ((delimitst == 1) && vp->flags.has_tag) {
 				/* Tagged attribute: print delimter and ignore tag */
 				buf[0] = '"';
-				librad_safeprint(vp->vp_strvalue,
+				fr_print_string(vp->vp_strvalue,
 						 vp->length, buf + 1, sizeof(buf) - 2);
 				strcat(buf, "\"");
 			} else if (delimitst == 1) {
 				/* Non-tagged attribute: print delimter */
 				buf[0] = '"';
-				librad_safeprint(vp->vp_strvalue,
+				fr_print_string(vp->vp_strvalue,
 						 vp->length, buf + 1, sizeof(buf) - 2);
 				strcat(buf, "\"");
 
@@ -233,7 +233,7 @@ int vp_prints_value(char * out, size_t outlen, VALUE_PAIR *vp, int delimitst)
 
 			} else {
 				/* Non-tagged attribute: no delimiter */
-				librad_safeprint(vp->vp_strvalue,
+				fr_print_string(vp->vp_strvalue,
 						 vp->length, buf, sizeof(buf));
 			}
 			a = buf;
@@ -272,6 +272,10 @@ int vp_prints_value(char * out, size_t outlen, VALUE_PAIR *vp, int delimitst)
 			}
 			if (len > 0) a = buf;
 			break;
+		case PW_TYPE_SIGNED: /* Damned code for 1 WiMAX attribute */
+			snprintf(buf, sizeof(buf), "%d", vp->vp_signed);
+			a = buf;
+			break;
 		case PW_TYPE_IPADDR:
 			a = inet_ntop(AF_INET, &(vp->vp_ipaddr),
 				      buf, sizeof(buf));
@@ -329,6 +333,15 @@ int vp_prints_value(char * out, size_t outlen, VALUE_PAIR *vp, int delimitst)
 			a = buf;
 			break;
 
+		case PW_TYPE_TLV:
+			if (outlen <= (2 * (vp->length + 1))) return 0;
+
+			strcpy(buf, "0t");
+
+			fr_bin2hex(vp->vp_tlv, buf + 2, vp->length);
+			a = buf;
+		  break;
+
 		default:
 			a = "UNKNOWN-TYPE";
 			break;
