@@ -1,7 +1,7 @@
 /*
  * detail.c	Process the detail file
  *
- * Version:	$Id: detail.c,v 1.14 2008/04/26 15:07:43 aland Exp $
+ * Version:	$Id$
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id: detail.c,v 1.14 2008/04/26 15:07:43 aland Exp $")
+RCSID("$Id$")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/modules.h>
@@ -39,6 +39,8 @@ RCSID("$Id: detail.c,v 1.14 2008/04/26 15:07:43 aland Exp $")
 
 #include <fcntl.h>
 
+#ifdef WITH_DETAIL
+
 #define USEC (1000000)
 
 typedef struct listen_detail_t {
@@ -210,6 +212,10 @@ static int detail_open(rad_listen_t *this)
 	 *	we've got to open it for writing in order to
 	 *	establish the lock, to prevent rlm_detail from
 	 *	writing to it.
+	 *
+	 *	This also means that if we're doing globbing,
+	 *	this file will be read && processed before the
+	 *	file globbing is done.
 	 */
 	this->fd = open(data->filename_work, O_RDWR);
 	if (this->fd < 0) {
@@ -226,7 +232,7 @@ static int detail_open(rad_listen_t *this)
 		if (stat(filename, &st) < 0) {
 #ifdef HAVE_GLOB_H
 			int i, found;
-			time_t ctime;
+			time_t chtime;
 			glob_t files;
 
 			memset(&files, 0, sizeof(files));
@@ -234,14 +240,14 @@ static int detail_open(rad_listen_t *this)
 				return 0;
 			}
 
-			ctime = 0;
+			chtime = 0;
 			found = -1;
 			for (i = 0; i < files.gl_pathc; i++) {
 				if (stat(files.gl_pathv[i], &st) < 0) continue;
 
 				if ((i == 0) ||
-				    (st.st_ctime < ctime)) {
-					ctime = st.st_ctime;
+				    (st.st_ctime < chtime)) {
+					chtime = st.st_ctime;
 					found = i;
 				}
 			}
@@ -253,8 +259,6 @@ static int detail_open(rad_listen_t *this)
 
 			filename = strdup(files.gl_pathv[found]);
 			globfree(&files);
-
-
 #else
 			return 0;
 #endif
@@ -783,7 +787,31 @@ int detail_parse(CONF_SECTION *cs, rad_listen_t *this)
 		return -1;
 	}
 
-	snprintf(buffer, sizeof(buffer), "%s.work", data->filename);
+	/*
+	 *	If the filename is a glob, use "detail.work" as the
+	 *	work file name.
+	 */
+	if ((strchr(data->filename, '*') != NULL) ||
+	    (strchr(data->filename, '[') != NULL)) {
+		char *p;
+
+#ifndef HAVE_GLOB_H
+		radlog(L_INFO, "WARNING: Detail file \"%s\" appears to use file globbing, but it is not supported on this system.", data->filename);
+#endif
+		strlcpy(buffer, data->filename, sizeof(buffer));
+		p = strrchr(buffer, FR_DIR_SEP);
+		if (p) {
+			p[1] = '\0';
+		} else {
+			buffer[0] = '\0';
+		}
+		strlcat(buffer, "detail.work",
+			sizeof(buffer) - strlen(buffer));
+			
+	} else {
+		snprintf(buffer, sizeof(buffer), "%s.work", data->filename);
+	}
+
 	free(data->filename_work);
 	data->filename_work = strdup(buffer); /* FIXME: leaked */
 
@@ -805,3 +833,4 @@ int detail_parse(CONF_SECTION *cs, rad_listen_t *this)
 
 	return 0;
 }
+#endif
