@@ -1,7 +1,7 @@
 /*
  * modules.c	Radius module support.
  *
- * Version:	$Id: modules.c,v 1.159 2008/04/20 15:00:07 aland Exp $
+ * Version:	$Id$
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id: modules.c,v 1.159 2008/04/20 15:00:07 aland Exp $")
+RCSID("$Id$")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/modpriv.h>
@@ -54,6 +54,13 @@ typedef struct section_type_value_t {
 	int		attr;
 } section_type_value_t;
 
+struct fr_module_hup_t {
+	module_instance_t	*mi;
+	time_t			when;
+	void			*insthandle;
+	fr_module_hup_t		*next;
+};
+
 
 /*
  *	Ordered by component
@@ -69,6 +76,64 @@ static const section_type_value_t section_type_value[RLM_COMPONENT_COUNT] = {
 	{ "post-auth",    "Post-Auth-Type",  PW_POST_AUTH_TYPE },
 };
 
+
+#ifdef WITHOUT_LIBLTDL
+typedef struct lt_dlmodule_t {
+  const char	*name;
+  void		*ref;
+} lt_dlmodule_t;
+
+/*
+ *	Define modules here.
+ */
+extern module_t rlm_pap;
+extern module_t rlm_chap;
+extern module_t rlm_eap;
+
+/*
+ *	EAP structures are defined elsewhere.
+ */
+typedef struct eap_type_t EAP_TYPE;
+
+/*
+ *	And so on for other EAP types.
+ */
+extern EAP_TYPE rlm_eap_md5;
+
+static const lt_dlmodule_t lt_dlmodules[] = {
+	{ "rlm_pap", &rlm_pap },
+	{ "rlm_chap", &rlm_chap },
+	{ "rlm_eap", &rlm_eap },
+	{ "rlm_eap_md5", &rlm_eap_md5 },
+	
+	/*
+	 *	Add other modules here.
+	 */
+		
+	{ NULL, NULL }
+};
+
+
+lt_dlhandle lt_dlopenext(const char *name)
+{
+	int i;
+
+	for (i = 0; lt_dlmodules[i].name != NULL; i++) {
+		if (strcmp(name, lt_dlmodules[i].name) == 0) {
+			return lt_dlmodules[i].ref;
+		}
+	}
+
+	return NULL;
+}
+
+void *lt_dlsym(lt_dlhandle handle, UNUSED const char *symbol)
+{
+	return handle;
+}
+#endif /* WITHOUT_LIBLTDL */
+
+
 static void indexed_modcallable_free(void *data)
 {
 	indexed_modcallable *c = data;
@@ -109,6 +174,38 @@ static int module_instance_cmp(const void *one, const void *two)
 }
 
 
+static void module_instance_free_old(CONF_SECTION *cs, module_instance_t *node,
+				     time_t when)
+{
+	fr_module_hup_t *mh, **last;
+
+	/*
+	 *	Walk the list, freeing up old instances.
+	 */
+	last = &(node->mh);
+	while (*last) {
+		mh = *last;
+
+		/*
+		 *	Free only every 60 seconds.
+		 */
+		if ((when - mh->when) < 60) {
+			last = &(mh->next);
+			continue;
+		}
+
+		cf_section_parse_free(cs, mh->insthandle);
+		
+		if (node->entry->module->detach) {
+			(node->entry->module->detach)(mh->insthandle);
+		}
+
+		*last = mh->next;
+		free(mh);
+	}
+}
+
+
 /*
  *	Free a module instance.
  */
@@ -116,15 +213,10 @@ static void module_instance_free(void *data)
 {
 	module_instance_t *this = data;
 
-	if (this->entry->module->detach) {
-		int i;
+	module_instance_free_old(this->cs, this, time(NULL) + 100);
 
+	if (this->entry->module->detach) {
 		(this->entry->module->detach)(this->insthandle);
-		for (i = 0; i < 16; i++) {
-			if (this->old_insthandle[i]) {
-				(this->entry->module->detach)(this->old_insthandle[i]);
-			}
-		}
 	}
 
 #ifdef HAVE_PTHREAD_H
@@ -273,7 +365,7 @@ static module_entry_t *linkto_module(const char *module_name,
  *	Find a module instance.
  */
 module_instance_t *find_module_instance(CONF_SECTION *modules,
-					const char *instname)
+					const char *instname, int do_link)
 {
 	CONF_SECTION *cs;
 	const char *name1, *name2;
@@ -301,6 +393,8 @@ module_instance_t *find_module_instance(CONF_SECTION *modules,
 	node = rbtree_finddata(instance_tree, &myNode);
 	if (node) return node;
 
+	if (!do_link) return NULL;
+
 	name1 = cf_section_name1(cs);
 	name2 = cf_section_name2(cs);
 
@@ -311,6 +405,7 @@ module_instance_t *find_module_instance(CONF_SECTION *modules,
 	memset(node, 0, sizeof(*node));
 
 	node->insthandle = NULL;
+	node->cs = cs;
 
 	/*
 	 *	Names in the "modules" section aren't prefixed
@@ -445,8 +540,8 @@ int indexed_modcall(int comp, int idx, REQUEST *request)
 
 	rcode = modcall(comp, list, request);
 
-	request->module = "<server-core>";
-	request->component = "<server-core>";
+	request->module = "";
+	request->component = "";
 	return rcode;
 }
 
@@ -532,7 +627,7 @@ static int define_type(const DICT_ATTR *dattr, const char *name)
 	} while (dict_valbyattr(dattr->attr, value));
 
 	if (dict_addvalue(name, dattr->name, value) < 0) {
-		radlog(L_ERR, "%s", librad_errstr);
+		radlog(L_ERR, "%s", fr_strerror());
 		return 0;
 	}
 
@@ -829,6 +924,51 @@ static int load_byserver(CONF_SECTION *cs)
 }
 
 
+int module_hup_module(CONF_SECTION *cs, module_instance_t *node, time_t when)
+{
+	void *insthandle = NULL;
+	fr_module_hup_t *mh;
+
+	if (!node ||
+	    !node->entry->module->instantiate ||
+	    ((node->entry->module->type & RLM_TYPE_HUP_SAFE) == 0)) {
+		return 1;
+	}
+
+	cf_log_module(cs, "Trying to reload module \"%s\"", node->name);
+	
+	if ((node->entry->module->instantiate)(cs, &insthandle) < 0) {
+		cf_log_err(cf_sectiontoitem(cs),
+			   "HUP failed for module \"%s\".  Using old configuration.",
+			   node->name);
+		return 0;
+	}
+
+	radlog(L_INFO, " Module: Reloaded module \"%s\"", node->name);
+
+	module_instance_free_old(cs, node, when);
+
+	/*
+	 *	Save the old instance handle for later deletion.
+	 */
+	mh = rad_malloc(sizeof(*mh));
+	mh->mi = node;
+	mh->when = when;
+	mh->insthandle = node->insthandle;
+	mh->next = node->mh;
+	node->mh = mh;
+
+	node->insthandle = insthandle;
+	
+	/*
+	 *	FIXME: Set a timeout to come back in 60s, so that
+	 *	we can pro-actively clean up the old instances.
+	 */
+
+	return 1;
+}
+
+
 int module_hup(CONF_SECTION *modules)
 {
 	time_t when;
@@ -836,16 +976,8 @@ int module_hup(CONF_SECTION *modules)
 	CONF_SECTION *cs;
 	module_instance_t *node;
 
-	static int insthandle_counter = -1;
-	static time_t hup_times[16];
-
 	if (!modules) return 0;
 
-	if (insthandle_counter < 0) {
-		memset(hup_times, 0, sizeof(hup_times));
-		insthandle_counter = 0;
-	}
-
 	when = time(NULL);
 
 	/*
@@ -854,8 +986,6 @@ int module_hup(CONF_SECTION *modules)
 	for (ci=cf_item_find_next(modules, NULL);
 	     ci != NULL;
 	     ci=cf_item_find_next(modules, ci)) {
-		int i;
-		void *insthandle = NULL;
 		const char *instname;
 		module_instance_t myNode;
 
@@ -870,67 +1000,10 @@ int module_hup(CONF_SECTION *modules)
 
 		strlcpy(myNode.name, instname, sizeof(myNode.name));
 		node = rbtree_finddata(instance_tree, &myNode);
-		if (!node ||
-		    !node->entry->module->instantiate ||
-		    ((node->entry->module->type & RLM_TYPE_HUP_SAFE) == 0)) {
-			continue;
-		}
 
-		cf_log_module(cs, "Trying to reload module \"%s\"", node->name);
-
-		if ((node->entry->module->instantiate)(cs, &insthandle) < 0) {
-			cf_log_err(cf_sectiontoitem(cs),
-				   "HUP failed for module \"%s\".  Using old configuration.",
-				   node->name);
-			continue;
-		}
-
-		radlog(L_INFO, " Module: Reloaded module \"%s\"", node->name);
-
-		/*
-		 *	Free all configurations old enough to be
-		 *	deleted.  This is slightly wasteful, but easy
-		 *	to do.
-		 *
-		 *	We permit HUPs every 5s, and we cache the last
-		 *	16 configurations.  So the current entry at
-		 *	insthandle_counter will either be empty, OR will
-		 *	be at least (5*16) = 80s old.  The following check
-		 *	ensures that it will be deleted.
-		 */
-		for (i = 0; i < 16; i++) {
-			if ((int) (when - hup_times[insthandle_counter]) < 60)
-				continue;
-
-			if (!node->old_insthandle[i]) continue;
-
-			cf_section_parse_free(cs, node->old_insthandle[i]);
-			
-			if (node->entry->module->detach) {
-				(node->entry->module->detach)(node->old_insthandle[i]);
-			}
-			node->old_insthandle[i] = NULL;
-		}
-
-		/*
-		 *	Save the old instance handle for later deletion.
-		 */
-		node->old_insthandle[insthandle_counter] = node->insthandle;
-		node->insthandle = insthandle;
-
-		/*
-		 *	FIXME: Set a timeout to come back in 60s, so that
-		 *	we can pro-actively clean up the old instances.
-		 */
+		module_hup_module(cs, node, when);
 	}
 
-	/*
-	 *	Remember when we were last HUP'd.
-	 */
-	hup_times[insthandle_counter] = when;
-	insthandle_counter++;
-	insthandle_counter &= 0x0f;
-
 	return 1;
 }
 
@@ -1045,7 +1118,7 @@ int setup_modules(int reload, CONF_SECTION *config)
 
 			cp = cf_itemtopair(ci);
 			name = cf_pair_attr(cp);
-			module = find_module_instance(modules, name);
+			module = find_module_instance(modules, name, 1);
 			if (!module) {
 				return -1;
 			}
@@ -1131,6 +1204,7 @@ int module_authenticate(int auth_type, REQUEST *request)
 	return indexed_modcall(RLM_COMPONENT_AUTH, auth_type, request);
 }
 
+#ifdef WITH_ACCOUNTING
 /*
  *	Do pre-accounting for ALL configured sessions
  */
@@ -1146,7 +1220,9 @@ int module_accounting(int acct_type, REQUEST *request)
 {
 	return indexed_modcall(RLM_COMPONENT_ACCT, acct_type, request);
 }
+#endif
 
+#ifdef WITH_SESSION_MGMT
 /*
  *	See if a user is already logged in.
  *
@@ -1172,7 +1248,9 @@ int module_checksimul(int sess_type, REQUEST *request, int maxsimul)
 
 	return (request->simul_count < maxsimul) ? 0 : request->simul_mpp;
 }
+#endif
 
+#ifdef WITH_PROXY
 /*
  *	Do pre-proxying for ALL configured sessions
  */
@@ -1188,6 +1266,7 @@ int module_post_proxy(int type, REQUEST *request)
 {
 	return indexed_modcall(RLM_COMPONENT_POST_PROXY, type, request);
 }
+#endif
 
 /*
  *	Do post-authentication for ALL configured sessions
