@@ -1,7 +1,7 @@
 /*
  * realms.c	Realm handling code
  *
- * Version:     $Id: realms.c,v 1.52 2008/04/18 09:29:49 aland Exp $
+ * Version:     $Id$
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id: realms.c,v 1.52 2008/04/18 09:29:49 aland Exp $")
+RCSID("$Id$")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/rad_assert.h>
@@ -65,11 +65,6 @@ static realm_regex_t *realms_regex = NULL;
 
 #endif /* HAVE_REGEX_H */
 
-static rbtree_t	*home_servers_byaddr = NULL;
-static rbtree_t	*home_servers_byname = NULL;
-
-static rbtree_t	*home_pools_byname = NULL;
-
 typedef struct realm_config_t {
 	CONF_SECTION	*cs;
 	int		dead_time;
@@ -81,6 +76,12 @@ typedef struct realm_config_t {
 
 static realm_config_t *realm_config = NULL;
 
+#ifdef WITH_PROXY
+static rbtree_t	*home_servers_byaddr = NULL;
+static rbtree_t	*home_servers_byname = NULL;
+
+static rbtree_t	*home_pools_byname = NULL;
+
 /*
  *  Map the proxy server configuration parameters to variables.
  */
@@ -107,6 +108,7 @@ static const CONF_PARSER proxy_config[] = {
 
 	{ NULL, -1, 0, NULL, NULL }
 };
+#endif
 
 static int realm_name_cmp(const void *one, const void *two)
 {
@@ -117,6 +119,7 @@ static int realm_name_cmp(const void *one, const void *two)
 }
 
 
+#ifdef WITH_PROXY
 static int home_server_name_cmp(const void *one, const void *two)
 {
 	const home_server *a = one;
@@ -147,7 +150,6 @@ static int home_server_addr_cmp(const void *one, const void *two)
 	return fr_ipaddr_cmp(&a->ipaddr, &b->ipaddr);
 }
 
-
 static int home_pool_name_cmp(const void *one, const void *two)
 {
 	const home_pool_t *a = one;
@@ -216,10 +218,11 @@ static size_t xlat_server_pool(UNUSED void *instance, REQUEST *request,
 
 	return strlen(out);
 }
-
+#endif
 
 void realms_free(void)
 {
+#ifdef WITH_PROXY
 	rbtree_free(home_servers_byname);
 	home_servers_byname = NULL;
 
@@ -228,6 +231,7 @@ void realms_free(void)
 
 	rbtree_free(home_pools_byname);
 	home_pools_byname = NULL;
+#endif
 
 	rbtree_free(realms_byname);
 	realms_byname = NULL;
@@ -248,6 +252,7 @@ void realms_free(void)
 }
 
 
+#ifdef WITH_PROXY
 static struct in_addr hs_ip4addr;
 static struct in6_addr hs_ip6addr;
 static char *hs_type = NULL;
@@ -281,7 +286,7 @@ static CONF_PARSER home_server_config[] = {
 	{ "status_check", PW_TYPE_STRING_PTR,
 	  0, &hs_check,   "none" },
 	{ "ping_check", PW_TYPE_STRING_PTR,
-	  0, &hs_check,   "none" },
+	  0, &hs_check,   NULL },
 
 	{ "ping_interval", PW_TYPE_INTEGER,
 	  offsetof(home_server,ping_interval), NULL,   "30" },
@@ -301,6 +306,11 @@ static CONF_PARSER home_server_config[] = {
 	{ "password",  PW_TYPE_STRING_PTR,
 	  offsetof(home_server,ping_user_password), NULL,  NULL},
 
+#ifdef WITH_STATS
+	{ "historic_average_window", PW_TYPE_INTEGER,
+	  offsetof(home_server,ema.window), NULL,  NULL },
+#endif
+
 	{ NULL, -1, 0, NULL, NULL }		/* end the list */
 
 };
@@ -488,7 +498,8 @@ static int home_server_add(realm_config_t *rc, CONF_SECTION *cs, int type)
 		}
 	}
 
-	if (rbtree_finddata(home_servers_byaddr, home)) {
+	if ((home->ipaddr.af != AF_UNSPEC) && /* could be virtual server */
+	    rbtree_finddata(home_servers_byaddr, home)) {
 		DEBUG2("Ignoring duplicate home server %s.", name2);
 		return 1;
 	}
@@ -501,7 +512,8 @@ static int home_server_add(realm_config_t *rc, CONF_SECTION *cs, int type)
 		return 0;
 	}
 
-	if (!rbtree_insert(home_servers_byaddr, home)) {
+	if ((home->ipaddr.af != AF_UNSPEC) && /* could be virtual server */
+	    !rbtree_insert(home_servers_byaddr, home)) {
 		rbtree_deletebydata(home_servers_byname, home);
 		cf_log_err(cf_sectiontoitem(cs),
 			   "Internal error adding home server %s.",
@@ -553,7 +565,8 @@ static int home_server_add(realm_config_t *rc, CONF_SECTION *cs, int type)
 			return 0;
 		}
 		
-		if (!rbtree_insert(home_servers_byaddr, home2)) {
+		if ((home->ipaddr.af != AF_UNSPEC) &&
+		    !rbtree_insert(home_servers_byaddr, home2)) {
 			rbtree_deletebydata(home_servers_byname, home2);
 			cf_log_err(cf_sectiontoitem(cs),
 				   "Internal error adding home server %s.",
@@ -684,6 +697,10 @@ static int server_pool_add(realm_config_t *rc,
 	pool->cs = cs;
 
 
+	/*
+	 *	Fallback servers must be defined, and must be
+	 *	virtual servers.
+	 */
 	cp = cf_pair_find(cs, "fallback");
 	if (cp) {
 		if (!pool_check_home_server(rc, cp, cf_pair_value(cp),
@@ -691,6 +708,11 @@ static int server_pool_add(realm_config_t *rc,
 			
 			goto error;
 		}
+
+		if (!pool->fallback->server) {
+			cf_log_err(cf_sectiontoitem(cs), "Fallback home_server %s does NOT contain a virtual_server directive.", pool->fallback->name);
+			goto error;
+		}
 	}
 
 	if (do_print) cf_log_info(cs, " home_server_pool %s {", name2);
@@ -789,7 +811,7 @@ static int server_pool_add(realm_config_t *rc,
 	free(pool);
 	return 0;
 }
-
+#endif
 
 static int old_server_add(realm_config_t *rc, CONF_SECTION *cs,
 			  const char *realm,
@@ -797,10 +819,19 @@ static int old_server_add(realm_config_t *rc, CONF_SECTION *cs,
 			  home_pool_type_t ldflag, home_pool_t **pool_p,
 			  int type, const char *server)
 {
+#ifdef WITH_PROXY
 	int i, insert_point, num_home_servers;
 	home_server myhome, *home;
 	home_pool_t mypool, *pool;
 	CONF_SECTION *subcs;
+#else
+	rc = rc;		/* -Wunused */
+	realm = realm;
+	secret = secret;
+	ldflag = ldflag;
+	type = type;
+	server = server;
+#endif
 
 	/*
 	 *	LOCAL realms get sanity checked, and nothing else happens.
@@ -813,6 +844,10 @@ static int old_server_add(realm_config_t *rc, CONF_SECTION *cs,
 		return 1;
 	}
 
+#ifndef WITH_PROXY
+	return 0;		/* Not proxying.  Can't do non-LOCAL realms */
+
+#else
 	mypool.name = realm;
 	mypool.server_type = type;
 	pool = rbtree_finddata(home_pools_byname, &mypool);
@@ -1020,6 +1055,7 @@ static int old_server_add(realm_config_t *rc, CONF_SECTION *cs,
 	*pool_p = pool;
 
 	return 1;
+#endif
 }
 
 static int old_realm_config(realm_config_t *rc, CONF_SECTION *cs, REALM *r)
@@ -1146,6 +1182,7 @@ static int old_realm_config(realm_config_t *rc, CONF_SECTION *cs, REALM *r)
 }
 
 
+#ifdef WITH_PROXY
 static int add_pool_to_realm(realm_config_t *rc, CONF_SECTION *cs,
 			     const char *name, home_pool_t **dest,
 			     int server_type, int do_print)
@@ -1192,6 +1229,7 @@ static int add_pool_to_realm(realm_config_t *rc, CONF_SECTION *cs,
 
 	return 1;
 }
+#endif
 
 
 static int realm_add(realm_config_t *rc, CONF_SECTION *cs)
@@ -1199,8 +1237,10 @@ static int realm_add(realm_config_t *rc, CONF_SECTION *cs)
 	const char *name2;
 	REALM *r = NULL;
 	CONF_PAIR *cp;
+#ifdef WITH_PROXY
 	home_pool_t *auth_pool, *acct_pool;
 	const char *auth_pool_name, *acct_pool_name;
+#endif
 
 	name2 = cf_section_name1(cs);
 	if (!name2 || (strcasecmp(name2, "realm") != 0)) {
@@ -1214,6 +1254,7 @@ static int realm_add(realm_config_t *rc, CONF_SECTION *cs)
 		return 0;
 	}
 
+#ifdef WITH_PROXY
 	auth_pool = acct_pool = NULL;
 	auth_pool_name = acct_pool_name = NULL;
 
@@ -1272,9 +1313,11 @@ static int realm_add(realm_config_t *rc, CONF_SECTION *cs)
 			return 0;
 		}
 	}
+#endif
 
 	cf_log_info(cs, " realm %s {", name2);
 
+#ifdef WITH_PROXY
 	/*
 	 *	The realm MAY already exist if it's an old-style realm.
 	 *	In that case, merge the old-style realm with this one.
@@ -1294,6 +1337,7 @@ static int realm_add(realm_config_t *rc, CONF_SECTION *cs)
 		cf_log_info(cs, " } # realm %s", name2);
 		return 1;
 	}
+#endif
 
 #ifdef HAVE_REGEX_H
 	if (name2[0] == '~') {
@@ -1316,10 +1360,11 @@ static int realm_add(realm_config_t *rc, CONF_SECTION *cs)
 	memset(r, 0, sizeof(*r));
 
 	r->name = name2;
+	r->striprealm = 1;
+#ifdef WITH_PROXY
 	r->auth_pool = auth_pool;
 	r->acct_pool = acct_pool;
-	r->striprealm = 1;
-
+	
 	if (auth_pool_name &&
 	    (auth_pool_name == acct_pool_name)) { /* yes, ptr comparison */
 		cf_log_info(cs, "\tpool = %s", auth_pool_name);
@@ -1327,6 +1372,7 @@ static int realm_add(realm_config_t *rc, CONF_SECTION *cs)
 		if (auth_pool_name) cf_log_info(cs, "\tauth_pool = %s", auth_pool_name);
 		if (acct_pool_name) cf_log_info(cs, "\tacct_pool = %s", acct_pool_name);
 	}
+#endif
 
 	cp = cf_pair_find(cs, "nostrip");
 	if (cp && (cf_pair_value(cp) == NULL)) {
@@ -1408,6 +1454,7 @@ int realms_init(CONF_SECTION *config)
 		return 0;
 	}
 
+#ifdef WITH_PROXY
 	home_servers_byaddr = rbtree_create(home_server_addr_cmp, free, 0);
 	if (!home_servers_byaddr) {
 		realms_free();
@@ -1425,11 +1472,13 @@ int realms_init(CONF_SECTION *config)
 		realms_free();
 		return 0;
 	}
+#endif
 
 	rc = rad_malloc(sizeof(*rc));
 	memset(rc, 0, sizeof(*rc));
 	rc->cs = config;
 
+#ifdef WITH_PROXY
 	cs = cf_subsection_find_next(config, NULL, "proxy");
 	if (cs) {
 		cf_section_parse(cs, rc, proxy_config);
@@ -1440,6 +1489,7 @@ int realms_init(CONF_SECTION *config)
 		rc->fallback = 0;
 		rc->wake_all_if_all_dead= 0;
 	}
+#endif
 
 	for (cs = cf_subsection_find_next(config, NULL, "realm");
 	     cs != NULL;
@@ -1451,8 +1501,10 @@ int realms_init(CONF_SECTION *config)
 		}
 	}
 
+#ifdef WITH_PROXY
 	xlat_register("home_server", xlat_home_server, NULL);
 	xlat_register("home_server_pool", xlat_server_pool, NULL);
+#endif
 
 	/*
 	 *	Swap pointers atomically.
@@ -1544,6 +1596,7 @@ REALM *realm_find(const char *name)
 }
 
 
+#ifdef WITH_PROXY
 home_server *home_server_ldb(const char *realmname,
 			     home_pool_t *pool, REQUEST *request)
 {
@@ -1645,7 +1698,7 @@ home_server *home_server_ldb(const char *realmname,
 			return home;
 		}
 
-		DEBUG3("PROXY %s %d\t%s %d",
+		RDEBUG3("PROXY %s %d\t%s %d",
 		       found->name, found->currently_outstanding,
 		       home->name, home->currently_outstanding);
 
@@ -1654,7 +1707,7 @@ home_server *home_server_ldb(const char *realmname,
 		 *	one we previously found.
 		 */
 		if (home->currently_outstanding < found->currently_outstanding) {
-			DEBUG3("Choosing %s: It's less busy than %s",
+			RDEBUG3("PROXY Choosing %s: It's less busy than %s",
 			       home->name, found->name);
 			found = home;
 			continue;
@@ -1665,20 +1718,7 @@ home_server *home_server_ldb(const char *realmname,
 		 *	we found.
 		 */
 		if (home->currently_outstanding > found->currently_outstanding) {
-			DEBUG3("Skipping %s: It's busier than %s",
-			       home->name, found->name);
-			continue;
-		}
-
-		if (home->total_requests_sent < found->total_requests_sent) {
-			DEBUG3("Choosing %s: It's been less busy than %s",
-			       home->name, found->name);
-			found = home;
-			continue;
-		}
-
-		if (home->total_requests_sent > found->total_requests_sent) {
-			DEBUG3("Skipping %s: It's been busier than %s",
+			RDEBUG3("PROXY Skipping %s: It's busier than %s",
 			       home->name, found->name);
 			continue;
 		}
@@ -1698,8 +1738,7 @@ home_server *home_server_ldb(const char *realmname,
 	/*
 	 *	There's a fallback if they're all dead.
 	 */
-	if (pool->fallback &&
-	    (pool->fallback->state == HOME_STATE_ALIVE)) {
+	if (pool->fallback) {
 		return pool->fallback;
 	}
 
@@ -1748,7 +1787,7 @@ home_server *home_server_ldb(const char *realmname,
 		}
 		if (!pool) return NULL;
 
-		DEBUG2("  Realm %s has no live home servers.  Falling back to the DEFAULT realm.", realmname);
+		RDEBUG2("PROXY - realm %s has no live home servers.  Falling back to the DEFAULT realm.", realmname);
 		return home_server_ldb(rd->name, pool, request);
 	}
 
@@ -1770,3 +1809,4 @@ home_server *home_server_find(fr_ipaddr_t *ipaddr, int port)
 
 	return rbtree_finddata(home_servers_byaddr, &myhome);
 }
+#endif
