@@ -1,7 +1,7 @@
 /*
  * threads.c	request threading support
  *
- * Version:	$Id: threads.c,v 1.142 2008/04/11 08:58:52 aland Exp $
+ * Version:	$Id$
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id: threads.c,v 1.142 2008/04/11 08:58:52 aland Exp $")
+RCSID("$Id$")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/rad_assert.h>
@@ -460,6 +460,13 @@ static void *request_handler_thread(void *arg)
 
 		DEBUG2("Thread %d got semaphore", self->thread_num);
 
+#ifdef HAVE_OPENSSL_ERR_H
+ 		/*
+		 *	Clear the error queue for the current thread.
+		 */
+		ERR_clear_error ();
+#endif
+
 		/*
 		 *	Try to grab a request from the queue.
 		 *
@@ -1111,10 +1118,19 @@ void thread_pool_unlock(void)
 {
 	pthread_mutex_unlock(&thread_pool.queue_mutex);
 }
-#else
-int thread_pool_addrequest(REQUEST *request, RAD_REQUEST_FUNP fun)
+
+void thread_pool_queue_stats(int *array)
 {
-	radius_handle_request(request, fun);
-	return 1;
+	int i;
+
+	if (pool_initialized) {
+		for (i = 0; i < RAD_LISTEN_MAX; i++) {
+			array[i] = fr_fifo_num_elements(thread_pool.fifo[i]);
+		}
+	} else {
+		for (i = 0; i < RAD_LISTEN_MAX; i++) {
+			array[i] = 0;
+		}
+	}
 }
 #endif /* HAVE_PTHREAD_H */
