@@ -15,20 +15,17 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  *
- * Copyright 2000  The FreeRADIUS server project
+ * Copyright 1999-2008  The FreeRADIUS server project
  * Copyright 2000  Alan DeKok <aland@ox.org>
  * Copyright 2000  Chris Parker <cparker@starnetusa.com>
  */
 
-static const char rcsid[] = "$Id$";
+#include <freeradius-devel/ident.h>
+RCSID("$Id$")
 
-#include "autoconf.h"
-
-#include <stdio.h>
-#include <stdlib.h>
-#include "radiusd.h"
+#include <freeradius-devel/radiusd.h>
 
 /*
  *	Display the revision number for this program
@@ -43,13 +40,10 @@ void NEVER_RETURNS version(void)
 	/* here are all the conditional feature flags */
 #if defined(OSFC2)
 	printf(" OSFC2");
-#endif
-#if defined(WITH_SNMP)
-	printf(" WITH_SNMP");
 #endif
 	printf("\n");
 #endif
-	printf("Copyright (C) 2000-2007 The FreeRADIUS server project.\n");
+	printf("Copyright (C) 1999-2008 The FreeRADIUS server project and contributors.\n");
 	printf("There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A\n");
 	printf("PARTICULAR PURPOSE.\n");
 	printf("You may redistribute copies of FreeRADIUS under the terms of the\n");
