@@ -1,7 +1,7 @@
 /*
  * rlm_attr_rewrite.c
  *
- * Version:  $Id: rlm_attr_rewrite.c,v 1.47 2008/02/09 08:00:55 aland Exp $
+ * Version:  $Id$
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id: rlm_attr_rewrite.c,v 1.47 2008/02/09 08:00:55 aland Exp $")
+RCSID("$Id$")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/modules.h>
@@ -395,7 +395,7 @@ do_again:
 		DEBUG2("%s: Changed value for attribute %s from '%s' to '%s'", data->name,
 				data->attribute, attr_vp->vp_strvalue, new_str);
 		if (pairparsevalue(attr_vp, new_str) == NULL) {
-			DEBUG2("%s: Could not write value '%s' into attribute %s: %s", data->name, new_str, data->attribute, librad_errstr);
+			DEBUG2("%s: Could not write value '%s' into attribute %s: %s", data->name, new_str, data->attribute, fr_strerror());
 			return ret;
 		}
 
