@@ -7,7 +7,7 @@
  * This code was written from scratch by Michael Richardson, and it is
  * dual licensed under both GPL and BSD.
  *
- * Version:     $Id: fips186prf.c,v 1.10 2007/11/23 13:17:22 aland Exp $
+ * Version:     $Id$
  *
  * GPL notice:
  *
@@ -45,7 +45,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id: fips186prf.c,v 1.10 2007/11/23 13:17:22 aland Exp $")
+RCSID("$Id$")
 
 #include <freeradius-devel/autoconf.h>
 
@@ -138,8 +138,12 @@ void fips186_2prf(uint8_t mk[20], uint8_t finalkey[160])
 
 		memset(zeros, 0, sizeof(zeros));
 		memcpy(zeros, xval.p, 20);
+#ifndef WITH_OPENSSL_SHA1
 		fr_SHA1Transform(context.state, zeros);
-		fr_fr_SHA1FinalNoLen(w_0.p, &context);
+#else
+		fr_SHA1Transform(&context, zeros);
+#endif
+		fr_SHA1FinalNoLen(w_0.p, &context);
 
 		/*   c. XKEY = (1 + XKEY + w_0) mod 2^160 */
 		onesixty_add_mod(&sum,  &xkey, &w_0);
@@ -153,8 +157,12 @@ void fips186_2prf(uint8_t mk[20], uint8_t finalkey[160])
 
 		memset(zeros, 0, sizeof(zeros));
 		memcpy(zeros, xval.p, 20);
+#ifndef WITH_OPENSSL_SHA1
 		fr_SHA1Transform(context.state, zeros);
-		fr_fr_SHA1FinalNoLen(w_1.p, &context);
+#else
+		fr_SHA1Transform(&context, zeros);
+#endif
+		fr_SHA1FinalNoLen(w_1.p, &context);
 
 		/*   f. XKEY = (1 + XKEY + w_1) mod 2^160 */
 		onesixty_add_mod(&sum,  &xkey, &w_1);
