@@ -1,7 +1,7 @@
 /*
  * rlm_eap.c  contains handles that are called from modules.
  *
- * Version:     $Id: rlm_eap.c,v 1.56 2008/03/11 08:19:41 aland Exp $
+ * Version:     $Id$
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id: rlm_eap.c,v 1.56 2008/03/11 08:19:41 aland Exp $")
+RCSID("$Id$")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/modules.h>
@@ -39,6 +39,8 @@ static const CONF_PARSER module_config[] = {
 	  offsetof(rlm_eap_t, ignore_unknown_eap_types), NULL, "no" },
 	{ "cisco_accounting_username_bug", PW_TYPE_BOOLEAN,
 	  offsetof(rlm_eap_t, cisco_accounting_username_bug), NULL, "no" },
+	{ "max_sessions", PW_TYPE_INTEGER,
+	  offsetof(rlm_eap_t, max_sessions), NULL, "2048"},
 
  	{ NULL, -1, 0, NULL, NULL }           /* end the list */
 };
@@ -135,7 +137,7 @@ static int eap_instantiate(CONF_SECTION *cs, void **instance)
 
 		eap_type = eaptype_name2type(auth_type);
 		if (eap_type < 0) {
-			radlog(L_ERR|L_CONS, "rlm_eap: Unknown EAP type %s",
+			radlog(L_ERR, "rlm_eap: Unknown EAP type %s",
 			       auth_type);
 			eap_detach(inst);
 			return -1;
@@ -157,7 +159,7 @@ static int eap_instantiate(CONF_SECTION *cs, void **instance)
 		if ((eap_type == PW_EAP_TLS) ||
 		    (eap_type == PW_EAP_TTLS) ||
 		    (eap_type == PW_EAP_PEAP)) {
-			DEBUG2("rlm_eap: Ignoring EAP-Type/%s because we do not have OpenSSL support.", auth_type);
+			DEBUG2("Ignoring EAP-Type/%s because we do not have OpenSSL support.", auth_type);
 			continue;
 		}
 #endif
@@ -250,7 +252,7 @@ static int eap_authenticate(void *instance, REQUEST *request)
 	 */
 	eap_packet = eap_vp2packet(request->packet->vps);
 	if (eap_packet == NULL) {
-		radlog(L_ERR, "rlm_eap: Malformed EAP Message");
+		radlog_request(L_ERR, 0, request, "Malformed EAP Message");
 		return RLM_MODULE_FAIL;
 	}
 
@@ -261,7 +263,7 @@ static int eap_authenticate(void *instance, REQUEST *request)
 	 */
 	handler = eap_handler(inst, &eap_packet, request);
 	if (handler == NULL) {
-		DEBUG2("  rlm_eap: Failed in handler");
+		RDEBUG2("Failed in handler");
 		return RLM_MODULE_INVALID;
 	}
 
@@ -277,7 +279,7 @@ static int eap_authenticate(void *instance, REQUEST *request)
 	if (rcode == EAP_INVALID) {
 		eap_fail(handler);
 		eap_handler_free(handler);
-		DEBUG2("  rlm_eap: Failed in EAP select");
+		RDEBUG2("Failed in EAP select");
 		return RLM_MODULE_INVALID;
 	}
 
@@ -285,7 +287,7 @@ static int eap_authenticate(void *instance, REQUEST *request)
 	 *	If we're doing horrible tunneling work, remember it.
 	 */
 	if ((request->options & RAD_REQUEST_OPTION_PROXY_EAP) != 0) {
-		DEBUG2("  Not-EAP proxy set.  Not composing EAP");
+		RDEBUG2("  Not-EAP proxy set.  Not composing EAP");
 		/*
 		 *	Add the handle to the proxied list, so that we
 		 *	can retrieve it in the post-proxy stage, and
@@ -343,7 +345,7 @@ static int eap_authenticate(void *instance, REQUEST *request)
 		 */
 		pairdelete(&request->proxy->vps, PW_FREERADIUS_PROXIED_TO);
 
-		DEBUG2("  Tunneled session will be proxied.  Not doing EAP.");
+		RDEBUG2("  Tunneled session will be proxied.  Not doing EAP.");
 		return RLM_MODULE_HANDLED;
 	}
 
@@ -374,6 +376,15 @@ static int eap_authenticate(void *instance, REQUEST *request)
 	     (handler->eap_ds->request->code == PW_EAP_SUCCESS) &&
 	     (handler->eap_ds->request->type.type == 0))) {
 
+		/*
+		 *	Return FAIL if we can't remember the handler.
+		 *	This is actually disallowed by the
+		 *	specification, as unexpected FAILs could have
+		 *	been forged.  However, we want to signal to
+		 *	everyone else involved that we are
+		 *	intentionally failing the session, as opposed
+		 *	to accidentally failing it.
+		 */
 		if (!eaplist_add(inst, handler)) {
 			eap_fail(handler);
 			eap_handler_free(handler);
@@ -381,7 +392,7 @@ static int eap_authenticate(void *instance, REQUEST *request)
 		}
 
 	} else {
-		DEBUG2("  rlm_eap: Freeing handler");
+		RDEBUG2("Freeing handler");
 		/* handler is not required any more, free it now */
 		eap_handler_free(handler);
 	}
@@ -515,7 +526,7 @@ static int eap_post_proxy(void *inst, REQUEST *request)
 							      request->proxy,
 							      REQUEST_DATA_EAP_TUNNEL_CALLBACK);
 		if (!data) {
-			radlog(L_ERR, "rlm_eap: Failed to retrieve callback for tunneled session!");
+			radlog_request(L_ERR, 0, request, "Failed to retrieve callback for tunneled session!");
 			eap_handler_free(handler);
 			return RLM_MODULE_FAIL;
 		}
@@ -523,11 +534,11 @@ static int eap_post_proxy(void *inst, REQUEST *request)
 		/*
 		 *	Do the callback...
 		 */
-		DEBUG2("  rlm_eap: Doing post-proxy callback");
+		RDEBUG2("Doing post-proxy callback");
 		rcode = data->callback(handler, data->tls_session);
 		free(data);
 		if (rcode == 0) {
-			DEBUG2("  rlm_eap: Failed in post-proxy callback");
+			RDEBUG2("Failed in post-proxy callback");
 			eap_fail(handler);
 			eap_handler_free(handler);
 			return RLM_MODULE_REJECT;
@@ -552,7 +563,7 @@ static int eap_post_proxy(void *inst, REQUEST *request)
 			}
 			
 		} else {	/* couldn't have been LEAP, there's no tunnel */
-			DEBUG2("  rlm_eap: Freeing handler");
+			RDEBUG2("Freeing handler");
 			/* handler is not required any more, free it now */
 			eap_handler_free(handler);
 		}
@@ -578,7 +589,7 @@ static int eap_post_proxy(void *inst, REQUEST *request)
 
 		return RLM_MODULE_OK;
 	} else {
-		DEBUG2("  rlm_eap: No pre-existing handler found");
+		RDEBUG2("No pre-existing handler found");
 	}
 
 
@@ -619,7 +630,7 @@ static int eap_post_proxy(void *inst, REQUEST *request)
 	 *	The format is very specific.
 	 */
 	if (vp->length != 17 + 34) {
-		DEBUG2("  rlm_eap: Cisco-AVPair with leap:session-key has incorrect length %d: Expected %d",
+		RDEBUG2("Cisco-AVPair with leap:session-key has incorrect length %d: Expected %d",
 		       vp->length, 17 + 34);
 		return RLM_MODULE_NOOP;
 	}
