@@ -1,7 +1,7 @@
 /*
  * peap.c contains the interfaces that are called from eap
  *
- * Version:     $Id: peap.c,v 1.36 2008/03/07 09:53:26 aland Exp $
+ * Version:     $Id$
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id: peap.c,v 1.36 2008/03/07 09:53:26 aland Exp $")
+RCSID("$Id$")
 
 #include "eap_peap.h"
 
@@ -34,8 +34,9 @@ RCSID("$Id: peap.c,v 1.36 2008/03/07 09:53:26 aland Exp $")
 static int eappeap_failure(EAP_HANDLER *handler, tls_session_t *tls_session)
 {
 	uint8_t tlv_packet[11];
+	REQUEST *request = handler->request;
 
-	DEBUG2("  rlm_eap_peap: FAILURE");
+	RDEBUG2("FAILURE");
 
 	tlv_packet[0] = PW_EAP_REQUEST;
 	tlv_packet[1] = handler->eap_ds->response->id +1;
@@ -68,8 +69,9 @@ static int eappeap_failure(EAP_HANDLER *handler, tls_session_t *tls_session)
 static int eappeap_success(EAP_HANDLER *handler, tls_session_t *tls_session)
 {
 	uint8_t tlv_packet[11];
+	REQUEST *request = handler->request;
 
-	DEBUG2("  rlm_eap_peap: SUCCESS");
+	RDEBUG2("SUCCESS");
 
 	tlv_packet[0] = PW_EAP_REQUEST;
 	tlv_packet[1] = handler->eap_ds->response->id +1;
@@ -97,7 +99,8 @@ static int eappeap_success(EAP_HANDLER *handler, tls_session_t *tls_session)
 /*
  *	Verify the tunneled EAP message.
  */
-static int eapmessage_verify(const uint8_t *data, unsigned int data_len)
+static int eapmessage_verify(REQUEST *request,
+			     const uint8_t *data, unsigned int data_len)
 {
 	const eap_packet_t *eap_packet = (const eap_packet_t *) data;
 	uint8_t eap_type;
@@ -110,7 +113,7 @@ static int eapmessage_verify(const uint8_t *data, unsigned int data_len)
 	eap_type = *data;
 	switch (eap_type) {
 	case PW_EAP_IDENTITY:
-		DEBUG2("  rlm_eap_peap: Identity - %*s",
+		RDEBUG2("Identity - %*s",
 		       data_len - 1, data + 1);
 		return 1;
 		break;
@@ -122,10 +125,10 @@ static int eapmessage_verify(const uint8_t *data, unsigned int data_len)
 		 */
 	case PW_EAP_RESPONSE:
 		if (eap_packet->data[0] == PW_EAP_TLV) {
-			DEBUG2("  rlm_eap_peap: Received EAP-TLV response.");
+			RDEBUG2("Received EAP-TLV response.");
 			return 1;
 		}
-		DEBUG2("  rlm_eap_peap: Got something weird.");
+		RDEBUG2("Got something weird.");
 		break;
 
 
@@ -135,7 +138,7 @@ static int eapmessage_verify(const uint8_t *data, unsigned int data_len)
 		 */
 	case PW_EAP_MSCHAPV2:
 	default:
-		DEBUG2("  rlm_eap_peap: EAP type %s",
+		RDEBUG2("EAP type %s",
 		       eaptype_type2name(eap_type,
 					 buffer, sizeof(buffer)));
 		return 1;
@@ -148,7 +151,7 @@ static int eapmessage_verify(const uint8_t *data, unsigned int data_len)
 /*
  *	Convert a pseudo-EAP packet to a list of VALUE_PAIR's.
  */
-static VALUE_PAIR *eap2vp(EAP_DS *eap_ds,
+static VALUE_PAIR *eap2vp(REQUEST *request, EAP_DS *eap_ds,
 			  const uint8_t *data, size_t data_len)
 {
 	size_t total;
@@ -158,7 +161,7 @@ static VALUE_PAIR *eap2vp(EAP_DS *eap_ds,
 
 	vp = paircreate(PW_EAP_MESSAGE, PW_TYPE_OCTETS);
 	if (!vp) {
-		DEBUG2("  rlm_eap_peap: Failure in creating VP");
+		RDEBUG2("Failure in creating VP");
 		return NULL;
 	}
 
@@ -184,7 +187,7 @@ static VALUE_PAIR *eap2vp(EAP_DS *eap_ds,
 
 		vp = paircreate(PW_EAP_MESSAGE, PW_TYPE_OCTETS);
 		if (!vp) {
-			DEBUG2("  rlm_eap_peap: Failure in creating VP");
+			RDEBUG2("Failure in creating VP");
 			pairfree(&head);
 			return NULL;
 		}
@@ -262,7 +265,7 @@ static int vp2eap(tls_session_t *tls_session, VALUE_PAIR *vp)
 /*
  *	See if there's a TLV in the response.
  */
-static int eappeap_check_tlv(const uint8_t *data)
+static int eappeap_check_tlv(REQUEST *request, const uint8_t *data)
 {
 	const eap_packet_t *eap_packet = (const eap_packet_t *) data;
 
@@ -276,7 +279,7 @@ static int eappeap_check_tlv(const uint8_t *data)
 		}
 
 		if (data[10] == EAP_TLV_FAILURE) {
-			DEBUG2("  rlm_eap_peap: Client rejected our response.  The password is probably incorrect.");
+			RDEBUG2("Client rejected our response.  The password is probably incorrect.");
 			return 0;
 		}
 	}
@@ -289,24 +292,22 @@ static int eappeap_check_tlv(const uint8_t *data)
  *	Use a reply packet to determine what to do.
  */
 static int process_reply(EAP_HANDLER *handler, tls_session_t *tls_session,
-			 UNUSED REQUEST *request, RADIUS_PACKET *reply)
+			 REQUEST *request, RADIUS_PACKET *reply)
 {
 	int rcode = RLM_MODULE_REJECT;
 	VALUE_PAIR *vp;
 	peap_tunnel_t *t = tls_session->opaque;
 
-#ifndef NDEBUG
 	if ((debug_flag > 0) && fr_log_fp) {
-		fprintf(fr_log_fp, "  PEAP: Processing from tunneled session code %p %d\n",
-		       reply, reply->code);
+		RDEBUG("Got tunneled reply RADIUS code %d",
+		       reply->code);
 
 		debug_pair_list(reply->vps);
 	}
-#endif
 
 	switch (reply->code) {
 	case PW_AUTHENTICATION_ACK:
-		DEBUG2("  PEAP: Tunneled authentication was successful.");
+		RDEBUG2("Tunneled authentication was successful.");
 		t->status = PEAP_STATUS_SENT_TLV_SUCCESS;
 		eappeap_success(handler, tls_session);
 		rcode = RLM_MODULE_HANDLED;
@@ -319,7 +320,7 @@ static int process_reply(EAP_HANDLER *handler, tls_session_t *tls_session,
 		 *	tunneled user!
 		 */
 		if (t->use_tunneled_reply) {
-			DEBUG2("  Saving tunneled attributes for later");
+			RDEBUG2("Saving tunneled attributes for later");
 
 			/*
 			 *	Clean up the tunneled reply.
@@ -334,14 +335,14 @@ static int process_reply(EAP_HANDLER *handler, tls_session_t *tls_session,
 		break;
 
 	case PW_AUTHENTICATION_REJECT:
-		DEBUG2("  PEAP: Tunneled authentication was rejected.");
+		RDEBUG2("Tunneled authentication was rejected.");
 		t->status = PEAP_STATUS_SENT_TLV_FAILURE;
 		eappeap_failure(handler, tls_session);
 		rcode = RLM_MODULE_HANDLED;
 		break;
 
 	case PW_ACCESS_CHALLENGE:
-		DEBUG2("  PEAP: Got tunneled Access-Challenge");
+		RDEBUG2("Got tunneled Access-Challenge");
 
 		/*
 		 *	Keep the State attribute, if necessary.
@@ -367,7 +368,7 @@ static int process_reply(EAP_HANDLER *handler, tls_session_t *tls_session,
 		 *	an "ack" packet.
 		 */
 		if (t->home_access_accept && t->use_tunneled_reply) {
-			DEBUG2("  Saving tunneled attributes for later");
+			RDEBUG2("Saving tunneled attributes for later");
 
 			/*
 			 *	Clean up the tunneled reply.
@@ -392,7 +393,7 @@ static int process_reply(EAP_HANDLER *handler, tls_session_t *tls_session,
 		break;
 
 	default:
-		DEBUG2("  PEAP: Unknown RADIUS packet type %d: rejecting tunneled user", reply->code);
+		RDEBUG2("Unknown RADIUS packet type %d: rejecting tunneled user", reply->code);
 		rcode = RLM_MODULE_REJECT;
 		break;
 	}
@@ -408,9 +409,9 @@ static int eappeap_postproxy(EAP_HANDLER *handler, void *data)
 {
 	int rcode;
 	tls_session_t *tls_session = (tls_session_t *) data;
-	REQUEST *fake;
+	REQUEST *fake, *request = handler->request;
 
-	DEBUG2("  PEAP: Passing reply from proxy back into the tunnel.");
+	RDEBUG2("Passing reply from proxy back into the tunnel.");
 
 	/*
 	 *	If there was a fake request associated with the proxied
@@ -441,12 +442,16 @@ static int eappeap_postproxy(EAP_HANDLER *handler, void *data)
 		fake->reply = request->proxy_reply;
 		request->proxy_reply = NULL;
 
+		if ((debug_flag > 0) && fr_log_fp) {
+			fprintf(fr_log_fp, "server %s {\n", fake->server);
+		}
+
 		/*
 		 *	Perform a post-auth stage, which will get the EAP
 		 *	handler, too...
 		 */
 		fake->options &= ~RAD_REQUEST_OPTION_PROXY_EAP;
-		DEBUG2("  PEAP: Passing reply back for EAP-MS-CHAP-V2");
+		RDEBUG2("Passing reply back for EAP-MS-CHAP-V2");
 		rcode = module_post_proxy(0, fake);
 
 		/*
@@ -455,14 +460,14 @@ static int eappeap_postproxy(EAP_HANDLER *handler, void *data)
 		 */
 		rcode = rad_postauth(fake);
 
-#ifndef NDEBUG
 		if ((debug_flag > 0) && fr_log_fp) {
-			fprintf(fr_log_fp, "  PEAP: Final reply from tunneled session code %d\n",
+			fprintf(fr_log_fp, "} # server %s\n", fake->server);
+			
+			RDEBUG("Final reply from tunneled session code %d",
 			       fake->reply->code);
-
+		
 			debug_pair_list(fake->reply->vps);
 		}
-#endif
 
 		/*
 		 *	Terrible hacks.
@@ -479,13 +484,12 @@ static int eappeap_postproxy(EAP_HANDLER *handler, void *data)
 		switch (rcode) {
                 case RLM_MODULE_FAIL:
 			request_free(&fake);
-			eaptls_fail(handler->eap_ds, 0);
+			eaptls_fail(handler, 0);
 			return 0;
 			break;
 
                 default:  /* Don't Do Anything */
-			DEBUG2(" PEAP: Got reply %d",
-			       request->proxy_reply->code);
+			RDEBUG2("Got reply %d", request->proxy_reply->code);
 			break;
 		}
 	}
@@ -513,29 +517,29 @@ static int eappeap_postproxy(EAP_HANDLER *handler, void *data)
 
 	switch (rcode) {
 	case RLM_MODULE_REJECT:
-		DEBUG2("  PEAP: Reply was rejected");
-		eaptls_fail(handler->eap_ds, 0);
+		RDEBUG2("Reply was rejected");
+		eaptls_fail(handler, 0);
 		return 0;
 
 	case RLM_MODULE_HANDLED:
-		DEBUG2("  PEAP: Reply was handled");
+		RDEBUG2("Reply was handled");
 		eaptls_request(handler->eap_ds, tls_session);
 		return 1;
 
 	case RLM_MODULE_OK:
-		DEBUG2("  PEAP: Reply was OK");
-		eaptls_success(handler->eap_ds, 0);
-		eaptls_gen_mppe_keys(&handler->request->reply->vps,
-				     tls_session->ssl,
-				     "client EAP encryption");
-		return 1;
+		RDEBUG2("Reply was OK");
+
+		/*
+		 *	Success: Automatically return MPPE keys.
+		 */
+		return eaptls_success(handler, 0);
 
 	default:
-		DEBUG2("  PEAP: Reply was unknown.");
+		RDEBUG2("Reply was unknown.");
 		break;
 	}
 
-	eaptls_fail(handler->eap_ds, 0);
+	eaptls_fail(handler, 0);
 	return 0;
 }
 
@@ -555,7 +559,6 @@ static void my_request_free(void *data)
  */
 int eappeap_process(EAP_HANDLER *handler, tls_session_t *tls_session)
 {
-	int err;
 	peap_tunnel_t *t = tls_session->opaque;
 	REQUEST *fake;
 	VALUE_PAIR *vp;
@@ -569,18 +572,6 @@ int eappeap_process(EAP_HANDLER *handler, tls_session_t *tls_session)
 	REQUEST *request = handler->request;
 	EAP_DS *eap_ds = handler->eap_ds;
 
-	/*
-	 *	FIXME: if the SSL session says "want read", or
-	 *	similar, leave the data in the clean_out buffer.  This
-	 *	lets the application data be sent across multiple
-	 *	fragments.
-	 */
-	err = tls_handshake_recv(tls_session);
-	if (!err) {
-		DEBUG2(" rlm_eap_peap: Failed in SSL");
-		return RLM_MODULE_REJECT;
-	}
-
 	/*
 	 *	Just look at the buffer directly, without doing
 	 *	record_minus.  This lets us avoid another data copy.
@@ -602,25 +593,58 @@ int eappeap_process(EAP_HANDLER *handler, tls_session_t *tls_session)
 	}
 #endif
 
-	if (!eapmessage_verify(data, data_len)) {
-		DEBUG2("  rlm_eap_peap: Tunneled data is invalid.");
+	if (!eapmessage_verify(request, data, data_len)) {
+		RDEBUG2("Tunneled data is invalid.");
 		return RLM_MODULE_REJECT;
 	}
 
-
 	/*
 	 *	If we authenticated the user, then it's OK.
 	 */
 	if (t->status == PEAP_STATUS_SENT_TLV_SUCCESS) {
-		if (eappeap_check_tlv(data)) {
-			DEBUG2("  rlm_eap_peap: Success");
+		if (eappeap_check_tlv(request, data)) {
+			RDEBUG2("Success");
 			return RLM_MODULE_OK;
 		}
 
+		/*
+		 *	Otherwise, the client rejected the session
+		 *	resumption.  If the session is being re-used,
+		 *	we need to do a full authentication.
+		 *
+		 *	We do this by sending an EAP-Identity request
+		 *	inside of the PEAP tunnel.
+		 */
+		if ((t->session_resumption_state != PEAP_RESUMPTION_NO) &&
+		    SSL_session_reused(tls_session->ssl)) {
+			eap_packet_t eap_packet;
+			
+			RDEBUG2("Client rejected session resumption.  Re-starting full authentication");
+			eap_packet.code = PW_EAP_REQUEST;
+			eap_packet.id = handler->eap_ds->response->id + 1;
+			eap_packet.length[0] = 0;
+			eap_packet.length[1] = EAP_HEADER_LEN + 1;
+			eap_packet.data[0] = PW_EAP_IDENTITY;
+			
+			/*
+			 *	Mark session resumption status.
+			 */
+			t->status = 0;
+			t->session_resumption_state = PEAP_RESUMPTION_NO;
+			
+			(tls_session->record_plus)(&tls_session->clean_in,
+						   &eap_packet,
+						   sizeof(eap_packet));
+			tls_handshake_send(tls_session);
+			return RLM_MODULE_HANDLED;
+		}
+
 		return RLM_MODULE_REJECT;
 
-	} else if (t->status == PEAP_STATUS_SENT_TLV_FAILURE) {
-		DEBUG2("  rlm_eap_peap:  Had sent TLV failure.  User was rejected earlier in this session.");
+	}
+
+	if (t->status == PEAP_STATUS_SENT_TLV_FAILURE) {
+		RDEBUG2(" Had sent TLV failure.  User was rejected earlier in this session.");
 		return RLM_MODULE_REJECT;
 	}
 
@@ -628,20 +652,20 @@ int eappeap_process(EAP_HANDLER *handler, tls_session_t *tls_session)
 
 	rad_assert(fake->packet->vps == NULL);
 
-	fake->packet->vps = eap2vp(eap_ds, data, data_len);
+	fake->packet->vps = eap2vp(request, eap_ds, data, data_len);
 	if (!fake->packet->vps) {
 		request_free(&fake);
-		DEBUG2("  rlm_eap_peap: Unable to convert tunneled EAP packet to internal server data structures");
+		RDEBUG2("Unable to convert tunneled EAP packet to internal server data structures");
 		return PW_AUTHENTICATION_REJECT;
 	}
 
-#ifndef NDEBUG
 	if ((debug_flag > 0) && fr_log_fp) {
-		fprintf(fr_log_fp, "  PEAP: Got tunneled EAP-Message\n");
+		RDEBUG("Got tunnled request");
 		
 		debug_pair_list(fake->packet->vps);
+
+		fprintf(fr_log_fp, "server %s {\n", fake->server);
 	}
-#endif
 
 	/*
 	 *	Tell the request that it's a fake one.
@@ -777,15 +801,13 @@ int eappeap_process(EAP_HANDLER *handler, tls_session_t *tls_session)
 
 	} /* else fake->server == request->server */
 
-#ifndef NDEBUG
 	if ((debug_flag > 0) && fr_log_fp) {
-		fprintf(fr_log_fp, "  PEAP: Sending tunneled request\n");
+		fprintf(fr_log_fp, "Sending tunneled request\n");
 
 		debug_pair_list(fake->packet->vps);
 
 		fprintf(fr_log_fp, "server %s {\n", fake->server);
 	}
-#endif
 
 	/*
 	 *	Call authentication recursively, which will
@@ -797,16 +819,13 @@ int eappeap_process(EAP_HANDLER *handler, tls_session_t *tls_session)
 	 *	Note that we don't do *anything* with the reply
 	 *	attributes.
 	 */
-#ifndef NDEBUG
 	if ((debug_flag > 0) && fr_log_fp) {
 		fprintf(fr_log_fp, "} # server %s\n", fake->server);
 
-		fprintf(fr_log_fp, "  PEAP: Got tunneled reply RADIUS code %d\n",
-			fake->reply->code);
+		RDEBUG("Got tunneled reply code %d", fake->reply->code);
 		
 		debug_pair_list(fake->reply->vps);
 	}
-#endif
 
 	/*
 	 *	Decide what to do with the reply.
